//*****************************************
// AVR Geiger Counter mega168 Program
// CPU Clock:1MHz
//   iq[Yf[^@WāAOIV[^(8MHz)ɂj
// KCK[JE^[vO
//
// File Name:timer.c
//
//  2011/12/17 Taishi New
//  2012/ 3/23 Taishi Ver 1.30
//*****************************************

#include	<avr/interrupt.h>
#include	<avr/io.h>
#include	<avr/eeprom.h>

#include	"ATmega168_geiger.h"
#include	"gei_eeprom.h"
#include	"utl.h"
#include	"glb_dat.h"
#include	"a_d.h"
#include	"timer.h"

static	unsigned char	tim_1ms;						// 10mSPʂJE^[
static	unsigned char	tim_10ms;						// PbPʂJE^[

// **********************************************************
// ********************** Soft Timer ************************
// **********************************************************

volatile unsigned int	wait_i;
volatile unsigned int	wait_j;
volatile unsigned char	wait_us;
// ************************************
// Delay 16uS
// t: 16uS (max 255)
// ************************************
void delay_16us(unsigned char t)
{
	while(t-- != 0){
		wait_us++;				// 1uS Delay??
	}
}
//***************************
// Wait
//  tm : Wait Time(ms) max. 8000
// CPU Clock:1MHz
//***************************
void	delay_ms(int tm)
{

	tm *= 2;
	for(wait_i = 0; wait_i < tm; wait_i++){
		delay_16us(63);
	}
}

// **********************************************************
// ************************ Timer 1 *************************
// **********************************************************

//*****************************************
// Timer1 Initial
//*****************************************
void	Timer1_init(void)
{
	PRR &= ~(1 << PRTIM1);												// Timer 1 ғ
	delay_16us(100/16);													// POOr҂

	TCCR1B = 0x00;														// Timer1 Stop

	TCNT1 = 0;													// Timer1 Count Set to ZERO
	TCCR1A = ( (1 << COM1B1) | (1 << WGM11) | (1 << WGM10) );	// PWM TOP=OCR1A, OC1B:BOTTOM=H Match=L, 
	TCCR1B = ( (1 << WGM13) | (1 << WGM12) );					// A Comper Match

//*************************************************
// **** RC@kgkOWma@PORigp ****
//*************************************************
// PfƂPl̕RUuɃfW{iOabcWOOjőA
// @iRUu RTDQujۂ̕ƂĎgA
// @Pl[fW{őlɕ|B
//398V
	OCR1A  = 2000;												// 1MHz / 500Hz = 2000
	OCR1B  = hv_out;											// ON Time ( 1=1uS)

}

//*****************************************
// High Volt ON
// \ timer1  TCCR1BȊÕWX^̓ZbgĂKv
//*****************************************
void	HV_on(void)
{
	TCCR1B |= ( 1 << CS10 );									// Timer1 Prescaler Set  clk-io/1
}
//*****************************************
// High Volt OFF
//*****************************************
void	HV_off(void)
{
	TCCR1B &= ~( (1 << CS12) | (1 << CS11) | (1 << CS10) );		// Timer1 Prescaler Stop
}
//*****************************************
// Set Timer1 ON Time
//*****************************************
void	set_Timer1_ontime(void)
{
	OCR1B  = hv_out;							// ON Time ( 1=1uS)
}
// **********************************************************
// ************************ Timer 0 *************************
// **********************************************************
//*****************************************
// Timer0 Initial
//*****************************************
void	Timer0_init(void)
{
	PRR &= ~(1 << PRTIM0);						// Timer 0 ғ
	delay_16us(100/16);							// POOr҂

	TCCR0B = 0;									// Timer Stop
	TCNT0 = 0;									// Timer0 Count Set to ZERO
	TCCR0A = ( (1 << COM0B1) | (1 << COM0B0) | ( 1 << WGM01) | ( 1 << WGM00) );	// ovl(A Comper Match)Aa]o(ON:Palus, OFF:Low)
	OCR0A  = KHz_4;								// 4kHz(250uS)
	OCR0B  = OCR0A/2;							// ̔

}

// **********************************************************
// ************************ Timer 2 *************************
// **********************************************************
//*****************************************
// Timer2 Initial
//  1mS 荞݂ɐݒ肷
//*****************************************
void	Timer2_init(void)
{
	PRR &= ~(1 << PRTIM2);								// Timer 1 ғ
	delay_16us(100/16);									// POOr҂

	TCCR2B = 0;											// Timer Stop
	TCNT2 = 0;											// Timer2 Count Set to ZERO
	TCCR2A = ((1 << WGM21) | (0 << WGM20));				// A Comper Match, Dont Output Timer2 Signal
	OCR2A  = COUNT_1MS;									// 1mS 8MHz/64=125kHz
	TCCR2B = ((0 << CS22) | (1 << CS21) | (0 << CS20));	// Timer Start (1MHz/8= 125kHz, 8uS)
	TIMSK2 |= (1 << OCIE2A);							// A Comper Match Interrupt Enable

	tim_1ms = 0;										// 10mSPʂJE^[
	tim_10ms = 0;										// PbPʂJE^[
	tim_1s_a = 60;										// PbPʃ^C}[
	tim_10ms_led = 0;									// kdcpPOrPʃ^C}[
	tim_10ms_ad0 = 0;									// `^cOpPOrPʃ^C}[

}
//***************************
// Timer2 Comper Match A Interrupt
//
//  1mS 
//***************************
ISR	(TIMER2_COMPA_vect)
{
	tim_1ms--;
	if(tim_1ms == 0){
		tim_1ms = 10;
		tim_10ms++;
		if(100 <= tim_10ms){					// Pbo
			tim_10ms = 0;
			if(tim_1s_a != 0){
				tim_1s_a--;						// PbPʃ^C}[@|P
			}
		}

		if(tim_10ms_led != 0){
			tim_10ms_led--;						// kdcpPOrPʃ^C}[@|P
		}

		if(tim_10ms_ad0 != 0){
			tim_10ms_ad0--;						// `^cOpPOrPʃ^C}[@|P
		}

		if(tim_10ms_sw != 0){
			tim_10ms_sw--;
		}

		if(tim_10ms_bat != 0){
			tim_10ms_bat--;						// obe[ቺ\pPOrPʃ^C}[
		}

		if(tim_10ms_swr_continue != 0){
			tim_10ms_swr_continue--;			// EXCb`씻ʗpPOrPʃ^C}[
		}

		if(tim_10ms_swr_length != 0){
			tim_10ms_swr_length--;				// EXCb`씻ʗpPOrPʃ^C}[
		}

		if(tim_10ms_swl_continue != 0){
			tim_10ms_swl_continue--;			// XCb`씻ʗpPOrPʃ^C}[
		}

		if(tim_10ms_swl_length != 0){
			tim_10ms_swl_length--;				// XCb`씻ʗpPOrPʃ^C}[
		}

		if(ad0_active == 1){					// `^cOgp̎
			ad0_set_data();						// A/D 0 Set A/D data to Data Buffer
		}
	}
}

